#!/usr/bin/env bash

# cp.sh - copy files to new location.
# Copyright (c) 2022 ge <https://nixhacks.net/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

tgt_cp() {
    # Transfer files to another location from __main_target_path using cp(1)
    # Handle 'file' URI scheme.
    #
    # Usage: tgt_cp URI

    log "Run handler ${FUNCNAME[0]}()"

    local uri
    local dst_path

    uri="$1"

    # shellcheck disable=SC2154
    if [[ "$uri" == "$__main_target" ]]; then
        : # Do nothing. Source and destination is the same
        log "Nothing to do:"\
            "Source and destination is the same: $__main_target"
        return
    fi

    # Copy backups to another destination
    parse_uri "$uri"

    # shellcheck disable=SC2154
    if [[ "$scheme" != 'file' ]]; then
        log -p "Error: Wrong URI scheme: $scheme" >&2; exit 1
    fi

    # shellcheck disable=SC2154
    if [ -f "$path" ] || [ -d "$path" ]; then
        dst_path="$path"
    else
        echo "Error: Path '$path' from URI '$uri' does not exists" >&2
        exit 1
    fi

    # Copy files preserving metadata
    # shellcheck disable=SC2154
    for backup in "${backups[@]}"; do
        log "Copying file $backup to $dst_path ..."
        set -- cp --archive "$backup" "$dst_path"

        log "Run command: $*"

        if "$@" 2>> "$log"; then
            : # Success
        else
            handle_error \
                "Error: Something went wrong when executing command:\n\t$*"
        fi
    done
}
